# Table of Contents

- [Introduction](introduction.md)
- [Quick Start](quick-start.md)
- [Installation](installation/installation.md)
  - [Supported Platforms](installation/supported-platforms.md)
  - [Installing a Binary Distribution](installation/installing-binary.md)
  - [Installing Nix from Source](installation/installing-source.md)
    - [Prerequisites](installation/prerequisites-source.md)
    - [Obtaining a Source Distribution](installation/obtaining-source.md)
    - [Building Nix from Source](installation/building-source.md)
  - [Security](installation/nix-security.md)
    - [Single-User Mode](installation/single-user.md)
    - [Multi-User Mode](installation/multi-user.md)
  - [Environment Variables](installation/env-variables.md)
  - [Upgrading Nix](installation/upgrading.md)
- [Package Management](package-management/package-management.md)
  - [Basic Package Management](package-management/basic-package-mgmt.md)
  - [Profiles](package-management/profiles.md)
  - [Garbage Collection](package-management/garbage-collection.md)
    - [Garbage Collector Roots](package-management/garbage-collector-roots.md)
  - [Channels](package-management/channels.md)
  - [Sharing Packages Between Machines](package-management/sharing-packages.md)
    - [Serving a Nix store via HTTP](package-management/binary-cache-substituter.md)
    - [Copying Closures via SSH](package-management/copy-closure.md)
    - [Serving a Nix store via SSH](package-management/ssh-substituter.md)
    - [Serving a Nix store via S3](package-management/s3-substituter.md)
- [Writing Nix Expressions](expressions/writing-nix-expressions.md)
  - [A Simple Nix Expression](expressions/simple-expression.md)
    - [Expression Syntax](expressions/expression-syntax.md)
    - [Build Script](expressions/build-script.md)
    - [Arguments and Variables](expressions/arguments-variables.md)
    - [Building and Testing](expressions/simple-building-testing.md)
    - [Generic Builder Syntax](expressions/generic-builder.md)
  - [Writing Nix Expressions](expressions/expression-language.md)
    - [Values](expressions/language-values.md)
    - [Language Constructs](expressions/language-constructs.md)
    - [Operators](expressions/language-operators.md)
    - [Derivations](expressions/derivations.md)
      - [Advanced Attributes](expressions/advanced-attributes.md)
    - [Built-in Constants](expressions/builtin-constants.md)
    - [Built-in Functions](expressions/builtins.md)
- [Advanced Topics](advanced-topics/advanced-topics.md)
  - [Remote Builds](advanced-topics/distributed-builds.md)
  - [Tuning Cores and Jobs](advanced-topics/cores-vs-jobs.md)
  - [Verifying Build Reproducibility](advanced-topics/diff-hook.md)
  - [Using the `post-build-hook`](advanced-topics/post-build-hook.md)
- [Command Reference](command-ref/command-ref.md)
  - [Common Options](command-ref/opt-common.md)
  - [Common Environment Variables](command-ref/env-common.md)
  - [Main Commands](command-ref/main-commands.md)
    - [nix-env](command-ref/nix-env.md)
    - [nix-build](command-ref/nix-build.md)
    - [nix-shell](command-ref/nix-shell.md)
    - [nix-store](command-ref/nix-store.md)
  - [Utilities](command-ref/utilities.md)
    - [nix-channel](command-ref/nix-channel.md)
    - [nix-collect-garbage](command-ref/nix-collect-garbage.md)
    - [nix-copy-closure](command-ref/nix-copy-closure.md)
    - [nix-daemon](command-ref/nix-daemon.md)
    - [nix-hash](command-ref/nix-hash.md)
    - [nix-instantiate](command-ref/nix-instantiate.md)
    - [nix-prefetch-url](command-ref/nix-prefetch-url.md)
  - [Experimental Commands](command-ref/experimental-commands.md)
    - [nix](command-ref/nix.md)
  - [Files](command-ref/files.md)
    - [nix.conf](command-ref/conf-file.md)
- [Glossary](glossary.md)
- [Hacking](hacking.md)
- [Release Notes](release-notes/release-notes.md)
  - [Release 2.3 (2019-09-04)](release-notes/rl-2.3.md)
  - [Release 2.2 (2019-01-11)](release-notes/rl-2.2.md)
  - [Release 2.1 (2018-09-02)](release-notes/rl-2.1.md)
  - [Release 2.0 (2018-02-22)](release-notes/rl-2.0.md)
  - [Release 1.11.10 (2017-06-12)](release-notes/rl-1.11.10.md)
  - [Release 1.11 (2016-01-19)](release-notes/rl-1.11.md)
  - [Release 1.10 (2015-09-03)](release-notes/rl-1.10.md)
  - [Release 1.9 (2015-06-12)](release-notes/rl-1.9.md)
  - [Release 1.8 (2014-12-14)](release-notes/rl-1.8.md)
  - [Release 1.7 (2014-04-11)](release-notes/rl-1.7.md)
  - [Release 1.6.1 (2013-10-28)](release-notes/rl-1.6.1.md)
  - [Release 1.6 (2013-09-10)](release-notes/rl-1.6.md)
  - [Release 1.5.2 (2013-05-13)](release-notes/rl-1.5.2.md)
  - [Release 1.5 (2013-02-27)](release-notes/rl-1.5.md)
  - [Release 1.4 (2013-02-26)](release-notes/rl-1.4.md)
  - [Release 1.3 (2013-01-04)](release-notes/rl-1.3.md)
  - [Release 1.2 (2012-12-06)](release-notes/rl-1.2.md)
  - [Release 1.1 (2012-07-18)](release-notes/rl-1.1.md)
  - [Release 1.0 (2012-05-11)](release-notes/rl-1.0.md)
  - [Release 0.16 (2010-08-17)](release-notes/rl-0.16.md)
  - [Release 0.15 (2010-03-17)](release-notes/rl-0.15.md)
  - [Release 0.14 (2010-02-04)](release-notes/rl-0.14.md)
  - [Release 0.13 (2009-11-05)](release-notes/rl-0.13.md)
  - [Release 0.12 (2008-11-20)](release-notes/rl-0.12.md)
  - [Release 0.11 (2007-12-31)](release-notes/rl-0.11.md)
  - [Release 0.10.1 (2006-10-11)](release-notes/rl-0.10.1.md)
  - [Release 0.10 (2006-10-06)](release-notes/rl-0.10.md)
  - [Release 0.9.2 (2005-09-21)](release-notes/rl-0.9.2.md)
  - [Release 0.9.1 (2005-09-20)](release-notes/rl-0.9.1.md)
  - [Release 0.9 (2005-09-16)](release-notes/rl-0.9.md)
  - [Release 0.8.1 (2005-04-13)](release-notes/rl-0.8.1.md)
  - [Release 0.8 (2005-04-11)](release-notes/rl-0.8.md)
  - [Release 0.7 (2005-01-12)](release-notes/rl-0.7.md)
  - [Release 0.6 (2004-11-14)](release-notes/rl-0.6.md)
  - [Release 0.5 and earlier](release-notes/rl-0.5.md)
